#include <os.h>
#include "screen.h"
#include "charmap.h"
#define SCREEN_CONTRAST_ADDR 0x900F0020

#define SCREEN_BASE_PTR     0xC0000010 
#define SCREEN_MODE_ADDR    0xC000001C
#define SCREEN_INT_ADDR     0xC0000020
unsigned char* SCREEN_BASE_ADDR = 0;

#define SCREEN4_SIZE    SCREEN_WIDTH*SCREEN_HEIGHT/2
#define SCREEN16_SIZE   SCREEN_WIDTH*SCREEN_HEIGHT*2

int mode_bits=4;
unsigned short int* addr=0;
int contrast=0;
int off_mode=0;

void initScr() {
  SCREEN_BASE_ADDR =*(unsigned short int**)SCREEN_BASE_PTR;
  addr = SCREEN_BASE_ADDR;
  if(!is_cx) {
    int mode = *(volatile unsigned*) SCREEN_MODE_ADDR;
    mode = mode&~0b1110;
    mode = mode|0b1000;
    *(volatile unsigned*) SCREEN_MODE_ADDR = mode;
  }
  mode_bits=16;
  contrast=*(volatile unsigned*) SCREEN_CONTRAST_ADDR;
}

unsigned int setContrast(unsigned int level) {
  unsigned int lowlevel = level &0b11111111;
  if(lowlevel>CONTRAST_MAX) lowlevel=CONTRAST_MAX;
  if(lowlevel<CONTRAST_MIN) lowlevel=CONTRAST_MIN;
  level = (level & ~0b11111111)|lowlevel;
  contrast=level;
  if(!is_cx || !off_mode)
    *(volatile unsigned*) SCREEN_CONTRAST_ADDR=level;
  return level;
}

unsigned short int* getScreen() {
  return addr;
}

void setScreen(unsigned short int* buf) {
  addr=buf;
  *(volatile unsigned*) SCREEN_BASE_PTR = buf;
}

void startScrMode16() {
  int mode = *(volatile unsigned*) SCREEN_MODE_ADDR;
  mode = mode&~0b1110;
  mode = mode|0b1000;
  *(volatile unsigned*) SCREEN_MODE_ADDR = mode;
  mode_bits=16;
}

void endScrMode16() {
  if(mode_bits==16) {
    int mode = *(volatile unsigned*) SCREEN_MODE_ADDR;
    mode = mode&~0b100001110;
    mode = mode|0b0100;
    *(volatile unsigned*) SCREEN_MODE_ADDR = mode;
    mode_bits=4;
    *(volatile unsigned*) SCREEN_BASE_PTR = SCREEN_BASE_ADDR;
    addr=SCREEN_BASE_ADDR;
  }
} 

void switchScrOffOn(int s) {
  if(!is_cx) {
    int mask = 0b100000000001;
    int mode = *(volatile unsigned*) SCREEN_MODE_ADDR;
    if(s) mode |= mask;
    else mode &= ~mask;
    *(volatile unsigned*) SCREEN_MODE_ADDR = mode;
    //return mode&mask;
  } else {
    if(s) off_mode=0;
    else off_mode=1;
    if(!off_mode) setContrast(contrast);
    else *(volatile unsigned*) SCREEN_CONTRAST_ADDR=(contrast&~0b11111111)|CONTRAST_MIN;
  }
}

void setBufPixel(unsigned short int* buf, int x, int y, unsigned short int color) {
  if(x >= 0 && x < SCREEN_WIDTH && y >= 0 && y < SCREEN_HEIGHT) buf[y*SCREEN_WIDTH+x]=color;
}

void scroll(unsigned short int* buf) {
  int size=320*240*2;
  unsigned short int* tempbuf = (unsigned char*) malloc(SCREEN_WIDTH*SCREEN_HEIGHT*2);
  int x, y;

  for(y=0;y<(SCREEN_HEIGHT-CHAR_HEIGHT);y++)
    for(x=0;x<SCREEN_WIDTH;x++)
      tempbuf[y*SCREEN_WIDTH+x]=buf[(y+CHAR_HEIGHT)*SCREEN_WIDTH+x];

  memcpy(buf, tempbuf, size);
  free(tempbuf);
}

void putBufChar(unsigned short int* buf, int x, int y, char ch, int trsp) {
  int i, j, pixelOn;
  for(i = 0; i < CHAR_HEIGHT; i++) {
    for(j = 0; j < CHAR_WIDTH; j++) {
      pixelOn = charMap_ascii[(unsigned char)ch][i] << j;
      pixelOn = pixelOn & 0x80;
      if (pixelOn) {
        if(trsp && is_cx) setBufPixel(buf,x + j, y + i, 0); 
        else setBufPixel(buf,x + j, y + i, 0xFFFF); 
      } else if(!trsp) {
        setBufPixel(buf,x + j, y + i, 0);
      }
    }
  } 
}

void clrBuf(unsigned char* buf) {
  int color=0;
  memset(buf,color,SCREEN16_SIZE);
}

void clrScr() {
  clrBuf((unsigned char *) addr);
}

void refreshScr(unsigned char* buf) {
  memcpy(addr,buf,SCREEN_WIDTH*SCREEN_HEIGHT*mode_bits/8);
}
